
/**
***
***     Program that reproducees the RV-DCC time-varying risk model estimates in Table 5 and portfolio shares in Table 7 of Leroux, Martin and St John.
***
***     The water data are loaded from the excel file precip_runoff_observed.xlsxare, Jan 1925 to Dec 2010.
***
**/

new;


library maxlik,pgraph;

graphset;
gausset;
maxset;


cls;

format /rd 10,8;



chdir c:\users\Toshiba\dropbox\Portfolio_V_A\Computer_programs;




/***      Read in the reservoir water data (January 1913 to December 2012)       ****/

    vls = reshape(error(0),9,1);


    y1 = xlsreadm("precip_runoff_observed.xlsx","c8:c1039",2,"");                 //     Catchment runoff for Melbourne (mms) per month (1925 to 2010)

    y2 = xlsreadm("precip_runoff_observed.xlsx","c4:c1035",1,"");                 //     Precipitation for Melbourne (mms) per month (1925 to 2010)


    y = ln(y1)~ ln(y2);     //      Variables expressed in natural logarithms



    dum_drought_kathryn = xlsreadm("precip_runoff_observed.xlsx","c152:c1183",3,"");          //     Drought dummy variable (=1 if there is a drought) from Jan 1925 to Dec 2010



//      Read in the daily rain data


    rain1  = xlsreadm("rain_087034_daily","a13515:b48578",1,"");        //     Lovely Banks (missing observations in this & following data sets have been converted to 999.99) 

    rain2  = xlsreadm("rain_087043_daily","a124:b35187",1,"");          //     Meredith

    rain3  = xlsreadm("rain_087053_daily","a11183:b46246",1,"");        //     Portarlington

    rain4  = xlsreadm("rain_086117_daily","a8281:b43344",1,"");         //     Toorourrong

    rain5  = xlsreadm("rain_086131_daily","a21491:b56554",1,"");        //     Yan Yean

    rain6  = xlsreadm("rain_088060_daily","a11308:b46371",1,"");        //     Wallaby Creek



//      Solve missing data problem in rain data

    rain = rain1[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain1 = rain;



    rain = rain2[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain2 = rain;



    rain = rain3[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain3 = rain;




    rain = rain4[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain4 = rain;




    rain = rain5[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain5 = rain;




    rain = rain6[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain6 = rain;



    rain = (rain1 + rain2 + rain3 + rain4 + rain5 + rain6)/6;       //      Average daily rainfall across the 6 sites


    rain_reshape = reshape(rain,1152,30)';


    rain_reshape = rain_reshape - meanc(rain_reshape)';


    rv = sumc( rain_reshape.^2 );                   //      Realised volatility of rain


    rv = trimr(rv,120,0);                           //  Start sample in Jan 1925 to line-up with reservoir data



//      Construct monthly dummy variables to deseasonalise the data

t = rows(y);


d1  = vec( ones(1,t/12) | zeros(12-1,t/12) );
d2  = zeros(1,1)  | trimr(d1,0,1);
d3  = zeros(2,1)  | trimr(d1,0,2);
d4  = zeros(3,1)  | trimr(d1,0,3);
d5  = zeros(4,1)  | trimr(d1,0,4);
d6  = zeros(5,1)  | trimr(d1,0,5);
d7  = zeros(6,1)  | trimr(d1,0,6);
d8  = zeros(7,1)  | trimr(d1,0,7);
d9  = zeros(8,1)  | trimr(d1,0,8);
d10 = zeros(9,1)  | trimr(d1,0,9);
d11 = zeros(10,1) | trimr(d1,0,10);
d12 = zeros(11,1) | trimr(d1,0,11);

sdum = d1 ~ d2 ~ d3 ~ d4 ~ d5 ~ d6 ~ d7 ~ d8 ~ d9 ~ d10 ~ d11 ~ d12;




//      Preliminary Diagnostic Tests


format /rd 10,4;


print "Preliminary Diagnostic Tests";
print "****************************";
print "";

//     Reservoirs   

toggle = 0.0;       //       Allow for seasonal dummies in the first stage regression

print "Reservoir (log) ar(1) test results";
call ar(y[.,1]);
print ""; 

print "Reservoir (log) arch(1) test results";
call arch(y[.,1]);
print ""; print "";




//     Harvesting  

toggle = 0.0;       //       Allow for seasonal dummies in the first stage regression

print "Harvesting (log) ar(1) test results";
call ar(y[.,2]);
print ""; 

print "Harvesting (log) arch(1) test results";
call arch(y[.,2]);
print ""; print "";



//     RV  

toggle = 0.0;       //       Allow for seasonal dummies in the first stage regression

print "Realized volatility ar(1) test results with seasonality";
call ar(rv);
print ""; 

toggle = 1.0;       //       Allow for seasonal dummies in the first stage regression

print "Realized volatility ar(1) test results without seasonality";
call ar(rv);
print ""; 





//      Estimate GARCH model of reservoir


    __title = "MLE Estimates of the GARCH Model of Reservoirs";

    _max_Options = { bfgs hetcon screen };
    


itoggle = 0.0;      //  Compute parameter estimates and standard errors with constraints

    _max_active = 1 | ones(11,1) | 1 | 1 | 1 ;  


theta0 = {
3.5573686916494967 
-0.4980205406574156 
-0.9194897595420655 
-0.9148198661058691 
-0.7925560867846406 
-0.3240113019332429 
0.1008781390680123 
0.5300299351256869 
0.8178467286277511 
0.8287278983141504 
0.6543507025887910 
0.3472748015166280 
0.2974781575182213 
0.6960093886592875 
0.4528965828917615 

};


theta1 = theta0';


mean1 = zeros(rows(y),1); var1 = zeros(rows(y),1); z1 = zeros(rows(y),1);


    { theta1,lnl1,gvec,cov,retcode } = maxprt(maxlik(y[.,1],0,&lnlt_garch,theta0));



print "";
print "";
print "Log-likelihood =" lnl1;
print "AIC           =" -2*lnl1+2*rows(theta1)/rows(y);



    itoggle = 1.0;      //  Compute parameter estimates and standard errors without constraints

    theta0 = theta1;

    theta0[13] = theta0[13]^2;

    theta0[14] = theta0[14]^2;

    theta0[15] = theta0[15]^2;


    { theta1,lnl1,gvec,cov,retcode } = maxprt(maxlik(y[.,1],0,&lnlt_garch,theta0));


    format /rd 10,3;



//      Estimate RV-GARCH model of rain

   
    __title = "MLE Estimates of the RV-GARCH Model of Rain";

    _max_Options = { bfgs hetcon screen };

    itoggle = 0.0;      //  Compute parameter estimates and standard errors with constraints

    _max_active = 1 | ones(11,1) | 1 | 1 | 1 | 1 | 1 | 1 | 1 ;  

    theta0 = {
3.9001055577657060 
-0.2676082769893019 
-0.4006121947524763 
-0.1583006469512227 
0.1182692172041251 
0.3022750062952246 
0.4276623996120818 
0.4988720836855204 
0.5104842891768737 
0.3886904787394193 
0.3351751324060430 
0.1902029832705938 

0.1878274237223099 
0.3872713148451382 
0.6170179867754131 
0.1168372297725601 

5.5326123646869956 
0.7237667245706968 
1.2332509095735498 

};


    theta2 = theta0';


    mean2 = zeros(rows(y),1); var2 = zeros(rows(y),1); z2 = zeros(rows(y),2);



    { theta2,lnl2,gvec,cov,retcode } = maxprt(maxlik(y[.,2],0,&lnlt_rvgarch,theta0));

print "";
print "";
print "Log-lielihood =" lnl2;
print "AIC           =" -2*lnl2+2*rows(theta2)/rows(y);



    itoggle = 1.0;      //  Compute parameter estimates and standard errors without constraints

    theta0 = theta2;

    theta0[13] = theta0[13]^2;

    theta0[14] = theta0[14]^2;

    theta0[15] = theta0[15]^2;

    theta0[16] = theta0[16]^2;


    { theta2,lnl2,gvec,cov,retcode } = maxprt(maxlik(y[.,2],0,&lnlt_rvgarch,theta0));



//      Combine all standardised residuals


    z = z1 ~ z2;        //  The order of the standardised residuals is: (i) reservoirs, (ii) stormwater =, (iii) RV measurement error


//      Combine means (reservoirs, stormwater)

    
    mean = mean1 ~ mean2;


//      Combine the variances (reservoirs, stormwater)

    
    var = var1 ~ var2;



    format /rd 10,3;




//      Estimate the bivariate DCC model with RV-GARCH on the rain equation


    cov_mat = zeros(t,3);       /**     Declare covariance matrix global      **/

    m_mat   = zeros(t,2);


    __title = "Bivariate DCC Model";

    _max_Options = { bfgs hetcon screen };
    
    itoggle = 0.0;      //  Compute parameter estimates and standard errors with constraints

    _max_active = 1 | 1 ;  


    theta0 =  -2 | 1 ;

    theta0 = {
   -4.4562761864 
    0.3265096246 
};


 
    theta3 = theta0';

    { theta3,lnl3,gvec,cov,retcode } = maxprt(maxlik(y,0,&lnlt,theta0));



//      Compute standard errors by the delta methods as the method of substitution does not generate standard errors as a result of numerical issues  


    itoggle = 1.0;      //  Compute parameter estimates and standard errors without constraints

    theta0 = theta3;

    theta0[1] = cdfn(theta0[1])*10^6;

    theta0[2] = cdfn(theta0[2]);



     _max_MaxIters = 0;


    { theta3,lnl3,gvec,cov,retcode } = maxprt(maxlik(y,0,&lnlt,theta0));



print ""; print "";
print "Log-likelihood (GARCH)   : reservoir    =" lnl1;
print "Log-likelihood (RV-GARCH): harvestested =" lnl2;
print "Log-likelihood (DCC)                    =" lnl3;
print "";
print "Total Log-likelihood                    =" lnl1+lnl2+lnl3;

print "AIC (Based on Total logL)               =" -2*(lnl1+lnl2+lnl3)+2*(rows(theta1)+rows(theta2)+rows(theta3))/rows(y);


print ""; print "";



//     Residual Diagnostics  

toggle = 1.0;       //       Do not allow for seasonal dummies in the first stage regression

print "Reservoir (log) residual ar(1) test results";
call ar(z[.,1]);
print ""; 

print "Reservoir (log) residual arch(1) test results";
call arch(z[.,1]);
print ""; print "";


toggle = 1.0;       //       Do not allow for seasonal dummies in the first stage regression

print "Harvesting (log) residual ar(1) test results";
call ar(z[.,2]);
print ""; 

print "Harvesting (log) residual arch(1) test results";
call arch(z[.,2]);
print ""; print "";


print "Covariance residual ar(1) test results";
call ar(z[.,1].*z[.,2]);
print ""; print "";


format /rd 10,3;

 

call lnlt(theta0',y);


    format /rd 10,3;
    

    trend = seqa(1925+1/12,1/12,rows(y)); 




    //      Convert means to levels

    mue1 = exp( mean[.,1] + 0.5*cov_mat[.,1] );           //      Average inflows per capacity
    
    mue2 = exp( mean[.,2] + 0.5*cov_mat[.,3] );           //      Average rain per capacity
    
    mue3 = 0.0;                                                        //     Average flow of manufactured water per capacity


    //      Convert variances and covariance to levels

    sig11 = exp( 2*mean[.,1] + cov_mat[.,1] ) .* (exp(cov_mat[.,1]) - 1);       //      Variance of reservoir

    sig22 = exp( 2*mean[.,2] + cov_mat[.,3] ) .* (exp(cov_mat[.,3]) - 1);       //      Variance of stormwater 

    sig12 = exp(   mean[.,1] + mean[.,2] + 0.5*(cov_mat[.,1]+cov_mat[.,3]) ) .* (exp( cov_mat[.,2]) - 1 )  ;       //      Covariance of reservoir and stormwater



    rho12 = sig12./sqrt(sig11.*sig22);              //      Correlation (in levels)



//      Compute optimal weights based on the Leroux-Martin (2016) model

s_r         = 1290000*0.65;
s_h         = 217*0.75;
s_m			= 150000;							/** annual desal capacity: 150,000ML  **/
s_total     = s_r + s_h + s_m;					/** total physical water stock **/
n_r         = 1;								/** number of reservoirs ***/
n_h         = 1;
n_m         = 1;


/*****  Supply Costs  ****/

capK_r		=  1165515000/1000;				/** total annual capital costs for reservoirs in $, adjusted for the fact that water units are in ML **/
capK_h		=  390600/1000;					/** total annual harvesting capital costs **/
capK_m		=  655500000/1000;				/** total annual manufactured water capital costs  ***/
opk_r		=  0.0;							/** constant per unit reservoir operating costs  **/
opk_h		=  0.29;						/** constant per unit harvesting operating costs; to internalise externality add -1.02 **/
opk_m       =  1.08;						/** constant per unit manufacturing costs externality + 0.18 **/
c_r			= capK_r/s_r + opk_r;			/**	total unit reservoir costs: 3.01/kL during crisis  **/
c_h			= capK_h/s_h + opk_h;			/**	total unit harvesting costs: 5.38/kL during crisis  **/
c_m			= capK_m/s_m + opk_m;			/**	total unit manufactued water costs:5.45/kL  during crisis **/

p			= 2.47;							/** water price **/

lam_r		= p*(capK_r + opk_r*s_r - opk_r)/(capK_r + opk_r*s_r)^2;				/** see equation (9), assuming constant operating costs **/	
lam_h		= p*(capK_h + opk_h*s_h - opk_h)/(capK_h + opk_h*s_h)^2;								
lam_m		= p*(capK_m + opk_m*s_m - opk_m)/(capK_m + opk_m*s_m)^2;								


w           = n_r*s_r*(p/c_r) + n_h*s_h*(p/c_h) + n_m*s_m*(p/c_m);          /**  	total water stock weighted by supply costs; equation (2)  **/ 

catchment_h = 3550000;				/**  3550000 in m^2 total stormwater catchment area   ***/
h_eff       = 0.10; 				/**  unit free; average harvesting efficiency of how much rainwater is captured by stormwater harvesting activities across all sites **/


delta       = 0.03;                 /**    	discount rate   (unit free)  **/ 
xi          = 0.02;                 /**     population growth rate **/ 
gam         = 3.5;             	    /**   	 Arrow Pratt CRRA parameter; gamma > 0 implies risk aversion, gamma > 1 implies high risk aversion; HL gamma>1 extreme RA  **/ 



//      Capacities

    catch1 = 1071.5;        //      Catchment size of reservoirs (expressed in km^2) and GL

    catch2 = 3.55*h_eff;          //      Catchment size of stormwater (expressed in km^2) and GL




//      Calculation of  shares


sigma_rh	= zeros(12,1);
k_r			= zeros(12,1);
k_h			= zeros(12,1);
k			= zeros(12,1);
a_r			= zeros(12,1);
a_h			= zeros(12,1);
gam_impl	= zeros(12,1);
theta_r		= zeros(12,1);
theta_h		= zeros(12,1);
theta_m		= zeros(12,1);

x			= zeros(12,1);
theta_total = zeros(12,1);
cap_r	    = zeros(12,1);
cap_h		= zeros(12,1);
cap_m		= zeros(12,1);
cap_total	= zeros(12,1);
tc_opt		= zeros(12,1);
tc_s		= zeros(12,1);
cost_av		= zeros(12,1);

theta_annual= zeros(rows(y)/12,3);
tc_annual   = zeros(rows(y)/12,1);


n = 80;      //      Used to estimate reservoirs over a longer period (measured in terms of years ie for n+1 year period)

n = 97;     //      Choose for the scenario where reservoir moments are based on the full dataset

n = 0;


j = 1+n;

do while j <= rows(y)/12;           //      Do loop over years

    mu_r = meanc( mue1[1+(j-1)*12-n*12:j*12] )*12*catch1*ones(12,1);

    sigma_r = sqrt( meanc( sig11[1+(j-1)*12-n*12:j*12] ) )*sqrt(12)*catch1*ones(12,1);

    mu_h    = mue2[1+(j-1)*12:j*12]*12*catch2;

    sigma_h = sqrt( sig22[1+(j-1)*12:j*12] )*12*catch2;


    mu_m = 0.4*s_m;


    rho = rho12[1+(j-1)*12:j*12];



    i = 1;              //      Do loop over months within a year

    do while i <= 12;


	    k_r[i]  = 1/((1-rho[i]^2)*((sigma_r[i]/s_r) + sigma_r[i]*lam_r*c_r/p)^2);			/** referred to as k_r in paper; equation (19)  **/
	    k_h[i]  = 1/((1-rho[i]^2)*((sigma_h[i]/s_h) + sigma_h[i]*lam_h*c_h/p)^2);		/** referred to as k_h in paper; equation (19)  **/
	    k[i]    =  rho[i]*(k_r[i]^0.5)*(k_h[i]^0.5);										/** referred to as k in paper; equation (19)  **/
	    a_r[i] 	= (mu_r[i]/s_r - mu_m/s_m) + mu_r[i]*lam_r*c_r/p + ((sigma_r[i]^2)/s_r)*lam_r*c_r/p - mu_m*lam_m*c_m/p;				/** referred to as a_r in paper; equation (11)  **/
	    a_h[i]	= (mu_h[i]/s_h - mu_m/s_m) + mu_h[i]*lam_h*c_h/p + ((sigma_h[i]^2)/s_h)*lam_h*c_h/p - mu_m*lam_m*c_m/p;			/** referred to as a_h in paper; equation (12)  **/


	    theta_robs  = (s_total-150000-150)/s_total;  			/** 150/(1812+150) desal capacity over total system capacity as per Melbourne Water report 2013 **/
	

	    theta_r[i] = (k_r[i]*a_r[i] - k[i]*a_h[i])*(1/gam);	
	    theta_h[i] = (k_h[i]*a_h[i] - k[i]*a_r[i])*(1/gam);	
	    theta_m[i] = 1 - theta_r[i] - theta_h[i];

    i = i+1;

    endo;

theta_annual[j,.] = meanc(theta_r) ~ meanc(theta_h) ~ (1 - meanc(theta_r) - meanc(theta_h)) ;



    if (1 - meanc(theta_r) - meanc(theta_h)) >= 0.0; 
    
        theta_annual[j,.] = meanc(theta_r) ~ meanc(theta_h) ~ (1 - meanc(theta_r) - meanc(theta_h)) ;

    else;
    
        theta_annual[j,.] = meanc(theta_r)/(meanc(theta_r)+meanc(theta_h)) ~ meanc(theta_h)/(meanc(theta_r)+meanc(theta_h)) ~ 0.0 ;
    
    endif;   
    

       tc_annual[j] = (theta_annual[j,1]*s_total*c_r + theta_annual[j,2]*s_total*c_h + theta_annual[j,3]*s_total*c_m)*(1000/1000000000);

    print 1924+j;
    print mu_r~sigma_r~mu_h~sigma_h;
    print "";

    j = j + 1;

endo;

  

    format /rd 12,2;

print "";
print "             Optimal Average Shares";
print "     Year         Reservoir  Harvested  Manufactured   Drought(year)     Portfolio cost";
print seqa(1925,1,rows(y)/12)~theta_annual~sumc(reshape(dum_drought_kathryn,t/12,12)')~tc_annual;
print "";
print " Average(pa) " meanc( selif(theta_annual,theta_annual[.,1].>0.0) )';     //  Compute average share over nonzero values
print " Median(pa)  " median( selif(theta_annual,theta_annual[.,1].>0.0) )';
print " SD(pa)      " stdc( selif(theta_annual,theta_annual[.,1].>0.0) )';     //  Compute average share over nonzero values
print "";
print "    Probability of using desal plant (% of years) =" 100*meanc(theta_annual[.,3] .gt 0.0);
print "";
print " Average portfolio cost      " meanc( selif(tc_annual,theta_annual[.,1].>0.0) )';    //  Compute average share over nonzero values
print ""; print "";


output off;

stop;





//     Log of the likelihood of the garch model     

    proc lnlt_garch(b,y);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,rv,tmp;

        n   = cols(y);          /**     Number of equations                                     **/

        lnl = zeros(t,1);       /**     Declare log likelihood                                  **/

        v   = stdc(y);          /**     Initialise the disturbance vector                       **/
        
        vc  = vcx(y);

        rv = zeros(rows(y),1);
        

    if itoggle == 0.0;

        c = b[13]^2;

        a = b[14]^2;

        d = b[15]^2;

    else;

        c = b[13];

        a = b[14];

        d = b[15];

    endif;
        
           
        i = 1;

        do while i <= rows(y);

            x_m = sdum[i,1:11]*b[2:12] ;
 
            if i == 1;
 
               m = b[1] + x_m ;                                                   /**     Update conditional mean                         **/
                             
               vc  = vcx(y);           /**     Initialise  conditional variance-covariance matrix     **/
                
            else;          

                m = b[1] + x_m;                                                   /**     Update conditional mean                         **/
              
                vc = c + a*v^2 + d*vc;                                  /**     Update conditional variance-covariance matrix   **/

            endif;

            mean1[i] = m';
                        
            v = y[i,.] - m;                                                /**     Update residuals  (2x1)                             **/

            var1[i] = vc;
  
            z1[i]  = v./sqrt(vc);

            
            lnl[i] = -0.5*n*ln(2*pi) - 0.5*ln(det(vc)) - 0.5*v'inv(vc)*v;   /**     Log of the likelihood at observation i          **/

            i = i + 1;

        endo;

  
        retp( real( lnl ) );

    endp;






//     Log of the likelihood of the rv-garch model     

    proc lnlt_rvgarch(b,y);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,tmp,alpha0,alpha1,w,delta0,delta1,sig2w,lnl_rv;

        n   = cols(y);          /**     Number of equations                                     **/

        lnl = zeros(t,1);       /**     Declare log likelihood                                  **/

        lnl_rv = zeros(t,1);       /**     Declare log likelihood of RV component of the log-likelihood                                 **/

        v   = stdc(y);          /**     Initialise the disturbance vector                       **/
  

    if itoggle == 0.0;

        c = b[13]^2;

        a = b[14]^2;

        d = b[15]^2;
 
        e = b[16]^2;
 
    else;

        c = b[13];

        a = b[14];

        d = b[15];

        e = b[16];

    endif;
               

        delta0 = b[17];
        
        delta1 = b[18];
        
        sig2w  = b[19]^2;


//    delta0 = 0;

//delta1 = 0;

//sig2w = 1;        
        
       
        i = 1;

        do while i <= rows(y);

            x_m = sdum[i,1:11]*b[2:12] ;
            

            if i == 1;
 
              m = b[1] + x_m ;                                                   /**     Update conditional mean                         **/
                             
               vc  = vcx(y);           /**     Initialise  conditional variance-covariance matrix     **/
                
            else;          

                m = b[1] + x_m;                                                   /**     Update conditional mean                         **/
              
                vc = c + a*v^2 + d*vc + e*ln(rv[i-1]);                                  /**     Update conditional variance-covariance matrix   **/

            endif;

            mean2[i] = m';

            
            v = y[i,.] - m;                                                /**     Update residuals  (2x1)                             **/


			w = ln(rv[i]) - delta0 - delta1*vc;                          //      RV disturbance


            var2[i] = vc;
            
  
            z2[i,1]  = v./sqrt(vc);

            z2[i,2]  = w./sqrt(sig2w);

            
            lnl[i] = -0.5*n*ln(2*pi) - 0.5*ln(det(vc)) - 0.5*v'inv(vc)*v;   //      Log of the likelihood at observation i   

			lnl_rv[i] = -0.5*ln(2*pi) - 0.5*ln(sig2w) - 0.5*w^2/sig2w;   //      RV log-likelihood component


            i = i + 1;

        endo;

//        retp( real( lnl  ) );

        retp( real( lnl + lnl_rv ) );

    endp;




//      DCC bivariate model with RV-garch on rain equation


    proc lnlt(b,y);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,s,r,alpha0,alpha1,beta1,q,qbar,w,delta0,delta1,sig2w,lnl_rv,lnl_var;

        n   = cols(y);          /**     Number of equations                                     **/

        lnl = zeros(t,1);       /**     Declare log likelihood                                  **/

        lnl_var = zeros(t,1);       /**     Declare log likelihood for GARCH component                                **/

        lnl_rv = zeros(t,1);       /**     Declare log likelihood fpr RV component                                **/


        v   = stdc(y);          /**     Initialise the disturbance vector                       **/
        
        vc  = vcx(y);
     

        x_m  = zeros(2,1);

        x_vc = zeros(2,2);

        m = zeros(2,1);
 

        qbar = vcx(z[.,1 2]);

 
    if itoggle == 0.0;
        
        alpha1 = cdfn(b[1]);
        
        beta1  = cdfn(b[2]);

    else;

        alpha1 = b[1]/10^6;

        beta1  = b[2];

    endif;

    
        i = 1;

        do while i <= t;

  
            if i == 1;


                q = qbar;
                

            else;          


                q  = (1 - alpha1 - beta1)*qbar + alpha1*z[i-1,1 2]'*z[i-1,1 2] + beta1*q ;


            endif;


            s  = diagrv(eye(n),sqrt(var[i,.]));                                                     /**     Diagonal matrix of conditional standard deviations  **/
 
            r  = inv( diagrv( eye(n),sqrt(diag(q)) ) )*q*inv( diagrv( eye(n),sqrt(diag(q)) ) );     /**     Conditional correlation matrix                      **/

            vc  = s*r*s;                                                                             /**     Update conditional variance-covariance matrix       **/

 

//            m_mat[i,.] = m';
  

            cov_mat[i,.] = vech(vc)';

            lnl[i] = - 0.5*ln(det(r)) - 0.5*z[i,1 2]*inv(r)*z[i,1 2]' + 0.5*z[i,1 2]*z[i,1 2]';             /**     Log of the likelihood pf dcc component at observation i              **/


            i = i + 1;

        endo;


        retp( real( lnl ) );

    endp;


/**     Procedure to test for first order AR      **/

proc ar(y);

     local v,x,b,e,t,r2,stat;


     //     First Stage

     if toggle .== 0.0;

        x = ones(rows(y),1)~sdum[.,1:11];

     else;

        x = ones(rows(y),1);

      endif;

     b = y/x;

     v = y - x*b;



     //     Second Stage

     y = trimr(v,1,0);


     if toggle .== 0.0;

        x = ones(rows(y),1) ~ trimr(v,0,1) ~ trimr(sdum[.,1:11],1,0);

     else;

        x = ones(rows(y),1)~ trimr(v,0,1);

     endif;


     b = y/x;

     e = y - x*b;


     t  = rows(y);                           
     r2 = (1 - e'e/(y-meanc(y))'(y-meanc(y)));
     stat = t*r2;

     print "  Regression estimates = " b';
     print "  Sample size (t)      = " t;
     print "  R2                   = " r2;
     print "  Statistic            = " stat;
     print "  p-value              = " cdfchic(stat,1);
     print "";

     retp(stat);

endp;





/**     Procedure to test for first order ARCH      **/

proc arch(y);

     local v,x,b,e,t,r2,arch;

     //     First Stage

     if toggle .== 0.0;

        x = ones(rows(y),1) ~ sdum[.,1:11];

     else;

        x = ones(rows(y),1);

     endif;


     b = y/x;

     v = y - x*b;


    //     Second Stage

     y = trimr(v,1,0).^2;

     x = ones(rows(y),1) ~ trimr(v,0,1).^2;


     b = y/x;

     e = y - x*b;

     t  = rows(y);                           
     r2 = (1 - e'e/(y-meanc(y))'(y-meanc(y)));
     arch = t*r2;

     print "  Regression estimates = " b';
     print "  Sample size (t)      = " t;
     print "  R2                   = " r2;
     print "  Statistic            = " arch;
     print "  p-value              = " cdfchic(arch,1);
     print "";

     retp(arch);

endp;


/**     Procedure to test for seasonality in the conditional variance      **/

proc arch_seasonal(y);

     local v,x,b,e,t,r2,ssr0,ssr1,stat;

     //     First Stage

     if toggle .== 0.0;

        x = ones(rows(y),1) ~ sdum[.,1:11];

     else;

        x = ones(rows(y),1);

     endif;


     b = y/x;

     v = y - x*b;


    //     Second Stage (include seasonal dummies in the variance equation)

     y = trimr(v,1,0).^2;

     x = ones(rows(y),1) ~ trimr(v,0,1).^2 ~  trimr(sdum[.,1:11],1,0);

     b = y/x;

     e = y - x*b;

     ssr1 = e'e;        //      Unrestricted sum of squares


    //     Third Stage (do include seasonal dummies in the variance equation)

     y = trimr(v,1,0).^2;

     x = ones(rows(y),1) ~ trimr(v,0,1).^2 ;

     b = y/x;

     e = y - x*b;

     ssr0 = e'e;        //      Restricted sum of squares


     stat = rows(y)*ln(ssr0/ssr1);

     print "  Statistic            = " stat;
     print "  p-value              = " cdfchic(stat,11);
     print "";
ssr0~ssr1;
     retp(stat);

endp;



stop;


