
/**
***
***     Program that reproducees the constant covariance risk model estimates in Table 5 and portfolio shares in Table 7 of Leroux, Martin and St John.
***
***     The water data are loaded from the excel file precip_runoff_observed.xlsxare, Jan 1925 to Dec 2010.
***
**/

new;


library maxlik,pgraph;

graphset;
gausset;
maxset;


cls;

format /rd 10,8;



chdir c:\users\Toshiba\dropbox\Portfolio_V_A\Computer_programs;




/***      Read in the reservoir water data (January 1913 to December 2012)       ****/

    vls = reshape(error(0),9,1);


    y1 = xlsreadm("precip_runoff_observed.xlsx","c8:c1039",2,"");                 //     Catchment runoff for Melbourne (mms) per month (1925 to 2010)

    y2 = xlsreadm("precip_runoff_observed.xlsx","c4:c1035",1,"");                 //     Precipitation for Melbourne (mms) per month (1925 to 2010)



    y = ln(y1)~ ln(y2);                                                             //      Variables expressed in natural logarithms



    dum_drought_kathryn = xlsreadm("precip_runoff_observed.xlsx","c152:c1183",3,"");          //     Drought dummy variable (=1 if there is a drought) from Jan 1925 to Dec 2010



//      Read in the daily rain data


    rain1  = xlsreadm("rain_087034_daily","a13515:b48578",1,"");        //     Lovely Banks (missing observations in this & following data sets have been converted to 999.99) 

    rain2  = xlsreadm("rain_087043_daily","a124:b35187",1,"");          //     Meredith

    rain3  = xlsreadm("rain_087053_daily","a11183:b46246",1,"");        //     Portarlington

    rain4  = xlsreadm("rain_086117_daily","a8281:b43344",1,"");         //     Toorourrong

    rain5  = xlsreadm("rain_086131_daily","a21491:b56554",1,"");        //     Yan Yean

    rain6  = xlsreadm("rain_088060_daily","a11308:b46371",1,"");        //     Wallaby Creek



//      Solve missing data problem in rain data

    rain = rain1[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain1 = rain;



    rain = rain2[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain2 = rain;



    rain = rain3[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain3 = rain;




    rain = rain4[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain4 = rain;




    rain = rain5[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain5 = rain;




    rain = rain6[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain6 = rain;



    rain = (rain1 + rain2 + rain3 + rain4 + rain5 + rain6)/6;       //      Average daily rainfall across the 6 sites


    rain_reshape = reshape(rain,1152,30)';


    rain_reshape = rain_reshape - meanc(rain_reshape)';


    rv = sumc( rain_reshape.^2 );                   //      Realised volatility of rain



//      Construct monthly dummy variables to deseasonalise the data

t = rows(y);


d1  = vec( ones(1,t/12) | zeros(12-1,t/12) );
d2  = zeros(1,1)  | trimr(d1,0,1);
d3  = zeros(2,1)  | trimr(d1,0,2);
d4  = zeros(3,1)  | trimr(d1,0,3);
d5  = zeros(4,1)  | trimr(d1,0,4);
d6  = zeros(5,1)  | trimr(d1,0,5);
d7  = zeros(6,1)  | trimr(d1,0,6);
d8  = zeros(7,1)  | trimr(d1,0,7);
d9  = zeros(8,1)  | trimr(d1,0,8);
d10 = zeros(9,1)  | trimr(d1,0,9);
d11 = zeros(10,1) | trimr(d1,0,10);
d12 = zeros(11,1) | trimr(d1,0,11);

sdum = d1 ~ d2 ~ d3 ~ d4 ~ d5 ~ d6 ~ d7 ~ d8 ~ d9 ~ d10 ~ d11 ~ d12;




//      Estimate GARCH model of reservoir


    __title = "MLE Estimates of the GARCH Model of Reservoirs";

    _max_Options = { bfgs hetcon screen };
 
    _max_active = 1 | ones(11,1) | 1  ;  


theta0 = {
3.4725856846836010 
-0.4674051623824674 
-0.8257337107631776 
-0.8514142840461479 
-0.6487719937807795 
-0.2397067243577690 
0.1739870929658065 
0.5605728550312734 
0.8534021859648867 
0.8504779692586131 
0.6817311432909021 
0.3545437474914117 
0.5224995613943759 

};


theta1 = theta0';


mean1 = zeros(rows(y),1); var1 = zeros(rows(y),1); z1 = zeros(rows(y),1);


    { theta1,lnl1,gvec,cov,retcode } = maxprt(maxlik(y[.,1],0,&lnlt_garch,theta0));


print "";
print "";
print "Log-likelihood =" lnl1;
print "AIC            =" -2*lnl1+2*rows(theta1)/rows(y);



//      Estimate RV-GARCH model of rain

   
    __title = "MLE Estimates of the RV-GARCH Model of Rain";

    _max_Options = { bhhh hetcon screen };

    _max_active = 1 | ones(11,1) | 1 | 1 | 1 ;  

    theta0 = {
3.8964095765311511 
-0.2764970955249611 
-0.3804363737816449 
-0.1363010403913864 
0.1788249140126554 
0.3063566536338633 
0.4242193521524718 
0.4968577439449013 
0.5101313985897059 
0.3673202380549837 
0.3180945125644788 
0.1798560523068991 
0.4891448835604395 

5

1

};


theta2 = theta0';


mean2 = zeros(rows(y),1); var2 = zeros(rows(y),1); z2 = zeros(rows(y),2);


    { theta2,lnl2,gvec,cov,retcode } = maxprt(maxlik(y[.,2],0,&lnlt_rvgarch,theta0));


print "";
print "";
print "Log-lielihood =" lnl2;
print "AIC           =" -2*lnl2+2*rows(theta2)/rows(y);




call lnlt_rvgarch(theta0',y[.,2]);



//      Combine all standardised residuals


    z = z1 ~ z2;        //  The order of the standardised residuals is: (i) reservoirs, (ii) stormwater, (iii) RV measurement error


//      Combine means (reservoirs, stormwater)

    
    mean = mean1 ~ mean2;


//      Combine the variances (reservoirs, stormwater)

    
    var = var1 ~ var2;



    format /rd 10,3;
    


//      Estimate the bivariate DCC model with RV-GARCH on the rain equation


    cov_mat = zeros(t,3);       /**     Declare covariance matrix global      **/

    m_mat   = zeros(t,2);


    __title = "Bivariate DCC Model";

    _max_Options = { bfgs hetcon screen };
    

    _max_active = 1  ;  


    theta0 =  -2  ;


    theta0 = { -4.6434647570398733 
             };


 
    theta3 = theta0';

    { theta3,lnl3,gvec,cov,retcode } = maxprt(maxlik(y,0,&lnlt,theta0));


print "";
print "";
print "Log-lielihood =" lnl3;
print "AIC           =" -2*lnl3+2*rows(theta3)/rows(y);




print ""; print "";
print "Log-likelihood (GARCH)   : reservoir    =" lnl1;
print "Log-likelihood (RV-GARCH): harvestested =" lnl2;
print "Log-likelihood (DCC)                    =" lnl3;
print "";
print "Total Log-likelihood                    =" lnl1+lnl2+lnl3;

print "AIC (Based on Total logL)               =" -2*(lnl1+lnl2+lnl3)+2*(rows(theta1)+rows(theta2)+rows(theta3))/rows(y);




    //      Convert means to levels

    mue1 = exp( mean[.,1] + 0.5*cov_mat[.,1] );           //      Average inflows per capacity
    
    mue2 = exp( mean[.,2] + 0.5*cov_mat[.,3] );           //      Average rain per capacity
    
    mue3 = 0.0;                                                        //     Average flow of manufactured water per capacity


    //      Convert variances and covariance to levels

    sig11 = exp( 2*mean[.,1] + cov_mat[.,1] ) .* (exp(cov_mat[.,1]) - 1);       //      Variance of reservoir

    sig22 = exp( 2*mean[.,2] + cov_mat[.,3] ) .* (exp(cov_mat[.,3]) - 1);       //      Variance of stormwater 

    sig12 = exp(   mean[.,1] + mean[.,2] + 0.5*(cov_mat[.,1]+cov_mat[.,3]) ) .* (exp( cov_mat[.,2]) - 1 )  ;       //      Covariance of reservoir and stormwater



    rho12 = sig12./sqrt(sig11.*sig22);              //      Correlation (in levels)





//      Compute optimal weights based on the Leroux-Martin (2016) model


s_r         = 1290000*0.65;
s_h         = 217*0.75;
s_m			= 150000;							/** annual desal capacity: 150,000ML  **/
s_total     = s_r + s_h + s_m;					/** total physical water stock **/
n_r         = 1;								/** number of reservoirs ***/
n_h         = 1;
n_m         = 1;


/*****  Supply Costs  ****/

capK_r		=  1165515000/1000;				/** total annual capital costs for reservoirs in $, adjusted for the fact that water units are in ML **/
capK_h		=  390600/1000;					/** total annual harvesting capital costs **/
capK_m		=  655500000/1000;				/** total annual manufactured water capital costs  ***/
opk_r		=  0.0;							/** constant per unit reservoir operating costs  **/
opk_h		=  0.29;						/** constant per unit harvesting operating costs; to internalise externality add -1.02 **/
opk_m       =  1.08;						/** constant per unit manufacturing costs externality + 0.18 **/
c_r			= capK_r/s_r + opk_r;			/**	total unit reservoir costs: 3.01/kL during crisis  **/
c_h			= capK_h/s_h + opk_h;			/**	total unit harvesting costs: 5.38/kL during crisis  **/
c_m			= capK_m/s_m + opk_m;			/**	total unit manufactued water costs:5.45/kL  during crisis **/

p			= 2.47;							/** water price **/

lam_r		= p*(capK_r + opk_r*s_r - opk_r)/(capK_r + opk_r*s_r)^2;				/** see equation (9), assuming constant operating costs **/	
lam_h		= p*(capK_h + opk_h*s_h - opk_h)/(capK_h + opk_h*s_h)^2;								
lam_m		= p*(capK_m + opk_m*s_m - opk_m)/(capK_m + opk_m*s_m)^2;								


w           = n_r*s_r*(p/c_r) + n_h*s_h*(p/c_h) + n_m*s_m*(p/c_m);          /**  	total water stock weighted by supply costs; equation (2)  **/ 

catchment_h = 3550000;				/**  3550000 in m^2 total stormwater catchment area   ***/
h_eff       = 0.10; 				/**  unit free; average harvesting efficiency of how much rainwater is captured by stormwater harvesting activities across all sites **/


delta       = 0.03;                 /**    	discount rate   (unit free)  **/ 
xi          = 0.02;                 /**     population growth rate **/ 
gam         = 3.5;             	    /**   	 Arrow Pratt CRRA parameter; gamma > 0 implies risk aversion, gamma > 1 implies high risk aversion; HL gamma>1 extreme RA  **/ 



//      Capacities

    catch1 = 1071.5;        //      Catchment size of reservoirs (expressed in km^2) and GL

    catch2 = 3.55*h_eff;          //      Catchment size of stormwater (expressed in km^2) and GL



//      Calculation of  shares


sigma_rh	= zeros(12,1);
k_r			= zeros(12,1);
k_h			= zeros(12,1);
k			= zeros(12,1);
a_r			= zeros(12,1);
a_h			= zeros(12,1);
gam_impl	= zeros(12,1);
theta_r		= zeros(12,1);
theta_h		= zeros(12,1);
theta_m		= zeros(12,1);

x			= zeros(12,1);
theta_total = zeros(12,1);
cap_r	    = zeros(12,1);
cap_h		= zeros(12,1);
cap_m		= zeros(12,1);
cap_total	= zeros(12,1);
tc_opt		= zeros(12,1);
tc_s		= zeros(12,1);
cost_av		= zeros(12,1);

theta_annual= zeros(rows(y)/12,3);
tc_annual   = zeros(rows(y)/12,1);


n = 80;      //      Used to estimate reservoirs over a longer period (measured in terms of years ie for n+1 year period)

n = 97;     //      Choose for the scenario where reservoir moments are based on the full dataset

n = 0;


j = 1+n;

do while j <= rows(y)/12;           //      Do loop over years

    mu_r = meanc( mue1[1+(j-1)*12-n*12:j*12] )*12*catch1*ones(12,1);

    sigma_r = sqrt( meanc( sig11[1+(j-1)*12-n*12:j*12] ) )*sqrt(12)*catch1*ones(12,1);

    mu_h    = mue2[1+(j-1)*12:j*12]*12*catch2;

    sigma_h = sqrt( sig22[1+(j-1)*12:j*12] )*12*catch2;


    mu_m = 0.4*s_m;


    rho = rho12[1+(j-1)*12:j*12];


    i = 1;              //      Do loop over months within a year

    do while i <= 12;


	    k_r[i]  = 1/((1-rho[i]^2)*((sigma_r[i]/s_r) + sigma_r[i]*lam_r*c_r/p)^2);			/** referred to as k_r in paper; equation (19)  **/
	    k_h[i]  = 1/((1-rho[i]^2)*((sigma_h[i]/s_h) + sigma_h[i]*lam_h*c_h/p)^2);		/** referred to as k_h in paper; equation (19)  **/
	    k[i]    =  rho[i]*(k_r[i]^0.5)*(k_h[i]^0.5);										/** referred to as k in paper; equation (19)  **/
	    a_r[i] 	= (mu_r[i]/s_r - mu_m/s_m) + mu_r[i]*lam_r*c_r/p + ((sigma_r[i]^2)/s_r)*lam_r*c_r/p - mu_m*lam_m*c_m/p;				/** referred to as a_r in paper; equation (11)  **/
	    a_h[i]	= (mu_h[i]/s_h - mu_m/s_m) + mu_h[i]*lam_h*c_h/p + ((sigma_h[i]^2)/s_h)*lam_h*c_h/p - mu_m*lam_m*c_m/p;			/** referred to as a_h in paper; equation (12)  **/


	    theta_robs  = (s_total-150000-150)/s_total;  			/** 150/(1812+150) desal capacity over total system capacity as per Melbourne Water report 2013 **/
	

	    theta_r[i] = (k_r[i]*a_r[i] - k[i]*a_h[i])*(1/gam);	
	    theta_h[i] = (k_h[i]*a_h[i] - k[i]*a_r[i])*(1/gam);	
	    theta_m[i] = 1 - theta_r[i] - theta_h[i];

    i = i+1;

    endo;

    theta_annual[j,.] = meanc(theta_r) ~ meanc(theta_h) ~ (1 - meanc(theta_r) - meanc(theta_h)) ;


//      Check positivity of shares

    if (1 - meanc(theta_r) - meanc(theta_h)) >= 0.0; 
    
        theta_annual[j,.] = meanc(theta_r) ~ meanc(theta_h) ~ (1 - meanc(theta_r) - meanc(theta_h)) ;

    else;
    
        theta_annual[j,.] = meanc(theta_r)/(meanc(theta_r)+meanc(theta_h)) ~ meanc(theta_h)/(meanc(theta_r)+meanc(theta_h)) ~ 0.0 ;
    
    endif;   
    

       tc_annual[j] = (theta_annual[j,1]*s_total*c_r + theta_annual[j,2]*s_total*c_h + theta_annual[j,3]*s_total*c_m)*(1000/1000000000);

print 1924+j;
print mu_r~sigma_r~mu_h~sigma_h;
print "";

j = j + 1;

endo;


    format /rd 12,2;

print "";
print "             Optimal Average Shares";
print "     Year         Reservoir  Harvested  Manufactured   Drought(year)     Portfolio cost";
print seqa(1925,1,rows(y)/12)~theta_annual~sumc(reshape(dum_drought_kathryn,t/12,12)')~tc_annual;
print "";
print " Average(pa) " meanc( selif(theta_annual,theta_annual[.,1].>0.0) )';     //  Compute average share over nonzero values
print " Median(pa)  " median( selif(theta_annual,theta_annual[.,1].>0.0) )';
print " SD(pa)      " stdc( selif(theta_annual,theta_annual[.,1].>0.0) )';     //  Compute average share over nonzero values
print "";
print "    Probability of using desal plant (% of years) =" 100*meanc(theta_annual[.,3] .gt 0.0);
print "";
print " Average portfolio cost      " meanc( selif(tc_annual,theta_annual[.,1].>0.0) )';    //  Compute average share over nonzero values
print ""; print "";


output off;

stop;





//     Log of the likelihood of the garch model     

    proc lnlt_garch(b,y);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,rv,tmp;

        n   = cols(y);          /**     Number of equations                                     **/

        lnl = zeros(t,1);       /**     Declare log likelihood                                  **/

        v   = stdc(y);          /**     Initialise the disturbance vector                       **/
        
        vc  = vcx(y);

        rv = zeros(rows(y),1);
     

        c = b[13];
       
           
        i = 1;

        do while i <= rows(y);

            x_m = sdum[i,1:11]*b[2:12] ;
 
            if i == 1;
 
               m = b[1] + x_m ;                                                   /**     Update conditional mean                         **/
                             
               vc  = vcx(y);           /**     Initialise  conditional variance-covariance matrix     **/
                
            else;          

                m = b[1] + x_m;                                                   /**     Update conditional mean                         **/
              
                vc = c^2 ;                                  /**     Constant variance-covariance matrix   **/

            endif;

            mean1[i] = m';
                        
            v = y[i,.] - m;                                                /**     Update residuals  (2x1)                             **/

            var1[i] = vc;
  
            z1[i]  = v./sqrt(vc);

            
            lnl[i] = -0.5*n*ln(2*pi) - 0.5*ln(det(vc)) - 0.5*v'inv(vc)*v;   /**     Log of the likelihood at observation i          **/

            i = i + 1;

        endo;

  
        retp( real( lnl ) );

    endp;






//     Log of the likelihood of the rv-garch model     

    proc lnlt_rvgarch(b,y);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,tmp,alpha0,alpha1,w,delta0,delta1,sig2w,lnl_rv;

        n   = cols(y);          /**     Number of equations                                     **/

        lnl = zeros(t,1);       /**     Declare log likelihood                                  **/

        lnl_rv = zeros(t,1);       /**     Declare log likelihood of RV component of the log-likelihood                                 **/

        v   = stdc(y);          /**     Initialise the disturbance vector                       **/
                

        c = b[13];

        delta0 = b[14];

        delta1 = 0.0;

        sig2w = b[15]^2;
       
       
        i = 1;

        do while i <= rows(y);

            x_m = sdum[i,1:11]*b[2:12] ;
            

            if i == 1;
 
              m = b[1] + x_m ;                                                   /**     Update conditional mean                         **/
                             
               vc  = vcx(y);           /**     Initialise  conditional variance-covariance matrix     **/
                
            else;          

                m = b[1] + x_m;                                                   /**     Update conditional mean                         **/
              
                vc = c^2 ;                                  /**     Constant variance-covariance matrix   **/

            endif;

            mean2[i] = m';

            
            v = y[i,.] - m;                                                /**     Update residuals  (2x1)                             **/


			w = ln(rv[i]) - delta0 - delta1*vc;                          //      RV disturbance



            var2[i] = vc;
            
  
            z2[i,1]  = v./sqrt(vc);

            
            lnl[i] = -0.5*n*ln(2*pi) - 0.5*ln(det(vc)) - 0.5*v'inv(vc)*v;   //      Log of the likelihood at observation i   

			lnl_rv[i] = -0.5*ln(2*pi) - 0.5*ln(sig2w) - 0.5*w^2/sig2w;   //      RV log-likelihood component


            i = i + 1;

        endo;

        retp( real( lnl + lnl_rv ) );

    endp;




//      DCC bivariate model with RV-garch on rain equation


    proc lnlt(b,y);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,s,r,alpha0,alpha1,beta1,q,qbar,w,delta0,delta1,sig2w,lnl_rv,lnl_var;

        n   = cols(y);          /**     Number of equations                                     **/

        lnl = zeros(t,1);       /**     Declare log likelihood                                  **/

        lnl_var = zeros(t,1);       /**     Declare log likelihood for GARCH component                                **/

        lnl_rv = zeros(t,1);       /**     Declare log likelihood fpr RV component                                **/


        v   = stdc(y);          /**     Initialise the disturbance vector                       **/
        
        vc  = vcx(y);
     

        x_m  = zeros(2,1);

        x_vc = zeros(2,2);

        m = zeros(2,1);
 

        qbar = vcx(z[.,1 2]);

    
        
        alpha1 = cdfn(b[1]);
        
        beta1  = 0.0;

      
        i = 1;

        do while i <= t;

  
            if i == 1;


                q = qbar;
                

            else;          


                q  = (1 - alpha1 - beta1)*qbar + alpha1*z[i-1,1 2]'*z[i-1,1 2] + beta1*q ;


            endif;


            s  = diagrv(eye(n),sqrt(var[i,.]));                                                     /**     Diagonal matrix of conditional standard deviations  **/
 
            r  = inv( diagrv( eye(n),sqrt(diag(q)) ) )*q*inv( diagrv( eye(n),sqrt(diag(q)) ) );     /**     Conditional correlation matrix                      **/

            vc  = s*r*s;                                                                             /**     Update conditional variance-covariance matrix       **/

            cov_mat[i,.] = vech(vc)';

            lnl[i] = - 0.5*ln(det(r)) - 0.5*z[i,1 2]*inv(r)*z[i,1 2]' + 0.5*z[i,1 2]*z[i,1 2]';             /**     Log of the likelihood pf dcc component at observation i              **/


            i = i + 1;

        endo;


        retp( real( lnl ) );

    endp;





//      Model to be simulated


    proc model(z);

        local lnl,vc,m,v,i,c,a,d,e,f,n,m1,m2,x_vc,x_m,s,r,alpha0,alpha1,beta1,q,qbar,w,delta0,delta1,sig2w,lnl_rv,lnl_var,cov,var,h1,h2,lrvs;

        n   = cols(z)-1;          /**     Number of equations                                     **/


        var = ones(rows(z),2);       /**     Declare variance vector                                **/
 
        cov = zeros(rows(z),3);       /**     Declare covariance matrix                                **/

 

        qbar = vcx(z[.,1 2]);

       
        i = 1;

        do while i <= rows(z);



            //      Simulate the variance (reservoir)
 
            if i == 1;


               h1  = vcx(y[.,1]);           /**     Initialise  conditional variance-covariance matrix     **/
                

            else;          


                h1 = theta1[13]^2 ;                                  /**     Update conditional variance-covariance matrix   **/


            endif;

         
            var[i,1] = h1;
            
            


            //      Simulate the variance (stormwater)
 
            if i == 1;


               h2  = vcx(y[.,2]);           /**     Initialise  conditional variance-covariance matrix     **/
                

            else;          

               h2 = theta2[13]^2 ;                                  /**     Update conditional variance-covariance matrix   **/


            endif;

            
            var[i,2] = h2;
            
     
        

        //      Simulate the covariance
        
        alpha1 = cdfn(theta3[1]);
        
        beta1 = 0.0;

      
                 if i == 1;


                q = qbar;
                

            else;          


                q  = (1 - alpha1 - beta1)*qbar + alpha1*z[i-1,1 2]'*z[i-1,1 2] + beta1*q ;


            endif;


            s  = diagrv(eye(n),sqrt(var[i,.]));                                                     /**     Diagonal matrix of conditional standard deviations  **/
 
            r  = inv( diagrv( eye(n),sqrt(diag(q)) ) )*q*inv( diagrv( eye(n),sqrt(diag(q)) ) );     /**     Conditional correlation matrix                      **/

            vc  = s*r*s;                                                                             /**     Update conditional variance-covariance matrix       **/
  
            cov[i,.] = vech(vc)';
  

            i = i + 1;

        endo;


        retp( cov  );

    endp;



stop;

