
/**
***
***     Program that computes various statistics in Tables 1 to 4 and plots Figures 1 to 4 of Leroux, Martin and St John.
***
***     The water data are loaded from the excel file precip_runoff_observed.xlsxare, Jan 1925 to Dec 2010.
***
**/

new;


library maxlik,pgraph;

graphset;
gausset;
maxset;


cls;

format /rd 10,3;



chdir c:\users\Toshiba\dropbox\Portfolio_V_A\Computer_programs;


/***      Read in the reservoir water data (January 1913 to December 2012)       ****/


    vls = reshape(error(0),9,1);


    y1 = xlsreadm("precip_runoff_observed.xlsx","c8:c1039",2,"");                 //     Catchment runoff for Melbourne (mms) per month (1925 to 2010)

    y2 = xlsreadm("precip_runoff_observed.xlsx","c4:c1035",1,"");                 //     Precipitation for Melbourne (mms) per month (1925 to 2010)




//    dum_drought_kathryn = xlsreadm("precip_runoff_observed.xlsx","c8:c1207",3,"");          //     Drought dummy variable (=1 if there is a drought)

//    dum_drought_kathryn = trimr(dum_drought_kathryn,24,24);



//      Read in the daily rain data


    rain1  = xlsreadm("rain_087034_daily","a13515:b48578",1,"");        //     Lovely Banks (missing observations in this & following data sets have been converted to 999.99) 

    rain2  = xlsreadm("rain_087043_daily","a124:b35187",1,"");        //     Meredith

    rain3  = xlsreadm("rain_087053_daily","a11183:b46246",1,"");        //     Portarlington

    rain4  = xlsreadm("rain_086117_daily","a8281:b43344",1,"");        //     Toorourrong

    rain5  = xlsreadm("rain_086131_daily","a21491:b56554",1,"");        //     Yan Yean

    rain6  = xlsreadm("rain_088060_daily","a11308:b46371",1,"");        //     Wallaby Creek



//      Solve missing data problem in rain data

    rain = rain1[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain1 = rain;



    rain = rain2[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain2 = rain;



    rain = rain3[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain3 = rain;




    rain = rain4[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain4 = rain;




    rain = rain5[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain5 = rain;




    rain = rain6[.,2];

    i = 2; 
    
    do while i<=rows(rain);
    
        if rain[i] == 99999.9; rain[i] = rain[i-1]; endif;     
        
        i = i + 1;
        
    endo;

    rain6 = rain;



    rain = (rain1 + rain2 + rain3 + rain4 + rain5 + rain6)/6;       //      Average daily rainfall across the 6 sites


    rain_reshape = reshape(rain,1152,30)';




//      Generate statistics

    y = y1 ~ y2;                        //      Reservoir and rain  
  

    z = (y - meanc(y)')./stdc(y)';      //      Standardise the data


print "         Reservoir   Harvested ";
print "*******************************";

print "Mean     " meanc(y)';

print "Median   " median(y)';

print "Min      " minc(y)';

print "Max      " maxc(y)';

print "SD       " stdc(y)';

print "Variance " (stdc(y).^2)';

print "Skewness " meanc(z.^3)';

print "Kurtosis " meanc(z.^4)';



//      Generate time plots

_pnum = 2;

_paxht = 0.18;

_ptitlht = 0.25;

begwind;

    window(2,2,0);

setwind(1);

    title("(a) Reservoir Inflows");

    xlabel("Year");

    ylabel("Inflows (mm per month)");

    xtics(1920,2020,10,0);

    ytics(0,300,50,0);

    xy(seqa(1925,1/12,rows(y)),y[.,1]);

nextwind;

    title("(b) Rainwater Harvesting");

    xlabel("Year");

    ylabel("Precipitation (mm per month)");

    xy(seqa(1925,1/12,rows(y)),y[.,2]);

endwind;




//      Generate histograms

graphset;

_pnum = 2;

_pnumht = 0.2;

_paxht = 0.2;

_ptitlht = 0.2;

begwind;

    window(2,2,0);

setwind(1);

    title("(a) Reservoir Inflows");

    xlabel(" ");

    ylabel("Frequency (%)");

    xtics(0,250,50,0);

    ytics(0,25,5,0);

    histp(y[.,1],seqa(0,10,25));

nextwind;

    title("(b) Rainwater Harvesting");

    ylabel("Frequency (%)");

    histp(y[.,2],seqa(0,10,25));

nextwind;

    title("(c) Reservoir Inflows (logs)");

    ylabel("Frequency (%)");

    xtics(0,6,1,0);

    ytics(0,25,5,0);

    histp(ln(y[.,1]),seqa(0,0.35,21));

nextwind;

    title("(d) Rainwater Harvesting (logs)");

    ylabel("Frequency (%)");

    histp(ln(y[.,2]),seqa(0,0.25,21));

endwind;




//      Generate kernel density plots


graphset;

_pnum = 2;

_paxht = 0.2;

begwind;

    window(2,2,0);

setwind(1);

    title("(a) Reservoir Inflows");

    ylabel("Frequency (%)");

    t = rows(y);

    h = 1.06*stdc(y[.,1])*t^(-1/5);


    yord = seqa(0.0,1,250);

    w = pdfn((yord - y[.,1]')/h)'/h;
   
    f = meanc(w); 

    xy(yord,f);


nextwind;

    title("(b) Rainwater Harvesting");

    ylabel("Frequency (%)");

    h = 1.06*stdc(y[.,2])*t^(-1/5);

    yord = seqa(0.0,1,250);

    w = pdfn((yord - y[.,2]')/h)'/h;
   
    f = meanc(w); 

    xy(yord,f);

nextwind;

    title("(c) Reservoir Inflows (logs)");

    ylabel("Frequency (%)");

    h = 1.06*stdc(ln(y[.,1]))*t^(-1/5);

    yord = seqa(0.0,0.025,250);

    w = pdfn((yord - ln(y[.,1])')/h)'/h;
   
    f = meanc(w); 

xy(yord,f);


nextwind;

    title("(d) Rainwater Harvesting (logs)");

    ylabel("Frequency (%)");

    h = 1.06*stdc(ln(y[.,2]))*t^(-1/5);

    yord = seqa(0.0,0.025,250);

    w = pdfn((yord - ln(y[.,2])')/h)'/h;
   
    f = meanc(w); 

xy(yord,f);

endwind;




//      Compute modal values

{ b1,m1,freq1 } = histp(y[.,1],seqa(0,10,25));

{ b2,m2,freq2 } = histp(y[.,2],seqa(0,10,25));

print "Reservoir histogram";
b1~m1~freq1;

print ""; print "";

print "Harvested histogram";
b1~m1~freq1;




//      Compute realised variance for harvested precipitation

    rain_reshape = reshape(rain,rows(y),30)';
    
    rain_reshape = rain_reshape - meanc(rain_reshape)';
    
    rv = sumc( rain_reshape.^2 );                   //      Realised variance of rain

    rvol = sqrt(rv);                                //      Realised volatility (standard deviation)


print "";
print "Mean of realized variance   = " meanc(rv);
print "Mean of realized volatility = " meanc( sqrt(rv) );
print ""; print "";




    graphset;

    _pnum = 2;
    
    xtics(1920,2020,10,0);

    xlabel("Year");

    ylabel("Realized Variance");

    xy(seqa(1925,1/12,rows(rv)),rv);



    ylabel("Realized Volatility");

    xy(seqa(1925,1/12,rows(rv)),rvol);





//      Calculate seasonal factors and extreme observations

    t = rows(y);

    d1  = vec( ones(1,t/12) | zeros(12-1,t/12) );
    d2  = zeros(1,1)  | trimr(d1,0,1);
    d3  = zeros(2,1)  | trimr(d1,0,2);
    d4  = zeros(3,1)  | trimr(d1,0,3);
    d5  = zeros(4,1)  | trimr(d1,0,4);
    d6  = zeros(5,1)  | trimr(d1,0,5);
    d7  = zeros(6,1)  | trimr(d1,0,6);
    d8  = zeros(7,1)  | trimr(d1,0,7);
    d9  = zeros(8,1)  | trimr(d1,0,8);
    d10 = zeros(9,1)  | trimr(d1,0,9);
    d11 = zeros(10,1) | trimr(d1,0,10);
    d12 = zeros(11,1) | trimr(d1,0,11);

    sdum = d1 ~ d2 ~ d3 ~ d4 ~ d5 ~ d6 ~ d7 ~ d8 ~ d9 ~ d10 ~ d11 ~ d12;

    sfactor1 = y[.,1]/sdum;

    sfactor2 = y[.,2]/sdum;

    print "Seasonal factors";
    print " Month Reservoirs    Harvesting";
    seqa(1,1,12)~(sfactor1/meanc(sfactor1))~(sfactor2/meanc(sfactor2));



    u1 = y[.,1] - sdum*sfactor1;

    u2 = y[.,2] - sdum*sfactor2;


    trend = seqa(1925,1/12,rows(y));
     
 
    print "Sorted reservoir and Sorted harvesting (standardized units)";   
    
    sortc(trend~u1/stdc(u1),2)~sortc(trend~u2/stdc(u2),2);
    
 
 
 //     Generate moving average correlations
 
 
    corrt = zeros(rows(y)-12+1,1);

    i = 1;

    do while i<= rows(corrt);
    
        cr = corrx( (y[i:11+i,.]) );
        
        corrt[i] = cr[1,2];
        
        i = i + 1; 
        
    endo;    
    
    cr = corrx( (y) );
    
    corrt_av = cr[1,2];  
    
    print "Average correlation: 1925-2010 =" corrt_av;  

    xlabel("Year");

    ylabel("Correlation");

    xy(seqa(1926,1/12,rows(corrt)),corrt~corrt_av*ones(rows(corrt),1));
    
output off;